#include "parallel.h"

void main()
{
   char             *Subroutine_name = "node_main():";
   int               Command;
   matrix_node_bin   Matrix;
   vector_node_bin   Vector;

   Dlu_info         *LU_Info;

   tbx_timer        *Timer;
   ZIP_MAILER       *Mailer;


   initialize_node(&Matrix, &Vector);

   Mailer = Matrix.Mailer;
   Zip_push(Mailer);

   Timer = new_tbx_timer();

   while ((Command = receive_command_node(Mailer)) != TERMINATE)
   {
      switch (Command)
      {
         case DO_LU:
            receive_matrix_node(Mailer, &Matrix);
            initialize_node_matrix_info(&Matrix, &LU_Info);
#if 0
            print("%s before initialize_node_vector_info()\n", Subroutine_name);
#endif
            initialize_node_vector_info(&Matrix, &Vector);
#if 0
            print("%s after initialize_node_vector_info()\n", Subroutine_name);
            print("Vector = 0x%lx\n", &Vector);
            print("%s Vector.b = 0x%lx, Vector.x = 0x%lx\n", Subroutine_name,
                  Vector.b, Vector.x);
            print("%s Vector.bound = %d\n", Subroutine_name, Vector.bound);
#endif
            unpack_unfactorized_matrix(&Matrix);

            print("Start LU factor\n");
            G2_sync();
            /* start timer */
            tbx_timer_start(Timer);

            CALL_METHOD(Matrix.Factorization)(LU_Info, NULL, NULL,
                                              XTRA(Matrix.Factorization));
            /* stop timer */
            tbx_timer_stop(Timer);

            /* print time */
            compute_time(Timer);

#ifdef DEBUG_main_node
            print_time(Timer);
#endif
            pack_factorized_matrix(&Matrix);
            transmit_matrix_node(Mailer, &Matrix);

            transmit_timing_node(Mailer, Timer);
            tbx_timer_reset(Timer);
            break;

         case DO_SOLVE:
#if 0
            receive_matrix_node(Mailer, &Matrix);

            /* load the matrices into the structure */
            unpack_factorized_matrix(&Matrix, LU_Info);
#endif
#if 0
            print("%s Vector.bund = %d\n", Subroutine_name, Vector.bound);
            print("%s Vector.b = 0x%lx, Vector.x = 0x%lx\n", Subroutine_name,
                  Vector.b, Vector.x);
            print("%s before receive_col_vector_node()\n", Subroutine_name);
#endif
            receive_col_vector_node(Mailer, &Vector);

            print("Start Solve\n");
            G2_sync();
            /* start timer */
            tbx_timer_start(Timer);

            CALL_METHOD(Matrix.Solve)(LU_Info, Vector.b, Vector.x,
                                      XTRA(Matrix.Solve));
            /* stop timer */
            tbx_timer_stop(Timer);

            transmit_row_vector_node(Mailer, &Vector);
#if 0
            clear_row_vector_node(&Vector);
            print("Matrix.num_element = %d\n", Matrix.num_element);
#endif
            /* print time */
            compute_time(Timer);

#ifdef DEBUG_main_node
            print_time(Timer);
#endif

            transmit_timing_node(Mailer, Timer);
            tbx_timer_reset(Timer);

            break;

         default:
            print("Unknown command!");
            exit(1);
      }
   }

   Zip_pop();
   zip_exit();
}
